class AttributeCannotBeOverriddenError(Exception):
    pass


class CannotReassignUnitError(Exception):
    pass


class FinalUnitError(Exception):
    pass


class MissingOutError(Exception):
    pass
